unit Main;

{#######################################################################################
Delphi UpdateLoader

Sendet ber RS232 Updates an den Bootloader "FastBoot" Version 2.1 von Peter Dannegger

Copyright (C) 2009 Leo-Andres Hofmann
Kontakt: leo.andres@gmx.de   |   webmaster@leo-andres.de

Die kommerzielle Nutzung, auch in Teilen, ist untersagt.

#######################################################################################}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, XPMan, ExtCtrls, ComCtrls, IniFiles, Registry, CPortCtl,
  CPort, Grids, Protokoll;

type
  TMainWindow = class(TForm)
    CopyNote:    TLabel;
    XPManifest:  TXPManifest;
    SettingsGroupBox: TGroupBox;
    UpdateGroupBox: TGroupBox;
    StatusGroupBox: TGroupBox;
    PasswordEdit: TLabeledEdit;
    BaudComboBox: TComboBox;
    ComComboBox: TComboBox;
    Label2:      TLabel;
    Label1:      TLabel;
    SelectInBtn: TButton;
    FilenameInEdit: TLabeledEdit;
    StartBtn:    TButton;
    ProgramChkBox: TCheckBox;
    VerifyChkBox: TCheckBox;
    Label3:      TLabel;
    ConnectProgressBar: TProgressBar;
    Label4:      TLabel;
    ProgramProgressBar: TProgressBar;
    Label6:      TLabel;
    VerifyProgressBar: TProgressBar;
    BufferProgressBar: TProgressBar;
    Label5:      TLabel;
    OpenDialog:  TOpenDialog;
    InfoGroupBox: TGroupBox;
    InfoMemo:    TMemo;
    LogMemo:     TMemo;
    HintLabel:   TLabel;
    HintFlashTimer: TTimer;
    ComPort:     TComPort;
    ShowProgramChkBox: TCheckBox;
    ProgramGroupBox: TGroupBox;
    ProgramGrid: TStringGrid;
    ShowCompleteMemoryChkBox: TCheckBox;
    ShowProgramProgressBar: TProgressBar;
    StartupTimer: TTimer;
    ComTerminal1: TComTerminal;
    procedure StartupTimerTimer(Sender: TObject);
    procedure ShowCompleteMemoryChkBoxClick(Sender: TObject);
    procedure ShowProgramChkBoxClick(Sender: TObject);
    procedure ProgramGridDrawCell(Sender: TObject; ACol, ARow: integer;
      Rect: TRect; State: TGridDrawState);
    procedure StartBtnClick(Sender: TObject);
    procedure ComPortTxEmpty(Sender: TObject);
    procedure ComPortRxBuf(Sender: TObject; const Buffer: PAnsiChar;
      Count: integer);
    procedure FilenameInEditChange(Sender: TObject);
    procedure SelectInBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure VerifyChkBoxClick(Sender: TObject);
    procedure ProgramChkBoxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HintFlashTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    type
    TBuffer = array of byte;
  var
    FIni: TMemIniFile;
    ReceiveMode, CRCStatus, AnswerResult, VerifySuccess: integer;
    TxBufferEmpty: boolean;
    InProgrammingMode: boolean;

    ProgramBuffer: TBuffer;
    Buffersize, Flashsize, BufferLastAddress: longint;
    ConnectionCRC: word;
    LastCommand, Answer: string;

    Connected, OneWire, ConnectionSuccess, CRCSupported, ReceiveAnswer,
    AnswerReceived, ContinueProgram, ProgSuccess, SupportsVerify,
    CommandReceived: boolean;

    procedure Log(Text: string; ShowAsMessageBox: boolean = False;
      AddToInfoMemo: boolean = False; AddToLogMemo: boolean = True);
    procedure FlashHint(Show: boolean; Text: string = ''; Speed: integer = 500);
    function LoadHexFile(Filename: string; var Buffer: TBuffer;
      BufferSize: longint; var BufferLastAddress: longint): integer;
    procedure ShowHexBuffer(var StringGrid: TStringGrid; var Buffer: TBuffer;
      var BufferLastAddress: longint; ShowCompleteBuffer: boolean = False);
    function SendChar(Transmit: char; TxWaitTime: Dword;
      WaitUntilTx: boolean = True): boolean;
    procedure CalculateCRC(Data: byte; var CRC: word);
    function SendCommand(HexCommand: string; TxWaitTime: Dword;
      WaitUntilTx: boolean = True): boolean;
    function SendStr(Text: string; TxWaitTime: Dword;
      WaitUntilTx: boolean = True): boolean;
    function Connect: integer;
    function CheckCRC: integer;
    function ReadInfos: boolean;
    function ReadVal(Command: string; Timeout: dword): integer;
    function ProgramDevice: integer;
    function VerifyDevice: integer;
  public
    { Public declarations }
  end;

var
  MainWindow: TMainWindow;

implementation

{$R *.dfm}

///////////////////////////////////////////////////////////////////////////////
{$REGION 'Hilfsfunktionen (Com-Namen auflisten, Buffer ausgeben/sortieren)'}

function CommExists(CommPort: integer): boolean;
var
  s: array[0..10] of char;
  i: cardinal;
  CommConfig: TCommConfig;
begin
  s := 'COM';
  StrCat(s, PChar(IntToStr(Commport)));
  StrCat(s, #0);
  i      := SizeOf(CommConfig);
  Result := GetDefaultCommConfig(s, CommConfig, i);
end;

function GetComNames: TStrings;
var
  KeyNames1, KeyNames2, KeyNames3: TStrings;
  Res:     TStringList;
  i, ii, iii: integer;
  s, Key0: string;
begin
  KeyNames1 := TStringList.Create;
  KeyNames2 := TStringList.Create;
  KeyNames3 := TStringList.Create;
  Key0 := '\SYSTEM\CurrentControlSet\Enum\';
  Res := TStringList.Create;
  with TRegistry.Create do
  begin
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKeyReadOnly(Key0) then
      begin
        GetKeyNames(KeyNames1);
        for i := 0 to KeyNames1.Count - 1 do
        begin
          OpenKeyReadOnly(Key0 + KeyNames1[i] + '\');
          GetKeyNames(KeyNames2);
          for ii := 0 to KeyNames2.Count - 1 do
          begin
            OpenKeyReadOnly(Key0 + KeyNames1[i] + '\' + KeyNames2[ii] + '\');
            GetKeyNames(KeyNames3);
            for iii := 0 to KeyNames3.Count - 1 do
            begin
              OpenKeyReadOnly(Key0 + KeyNames1[i] + '\' + KeyNames2[ii] +
                '\' + KeyNames3[iii] + '\');
              if KeyExists('Device Parameters') then
              begin
                s := ReadString('FriendlyName');
                OpenKeyReadOnly(Key0 + KeyNames1[i] + '\' + KeyNames2[ii] +
                  '\' + KeyNames3[iii] + '\Device Parameters\');
                if ValueExists('PortName') then
                begin
                  s := ReadString('PortName') + ': ' + s;
                  if Pos('COM', s) = 1 then
                    if CommExists(StrToInt(copy(s, 4, 1))) then
                      Res.Add(s + ' @ ' + KeyNames1[i]);
                end;
              end;
            end;
          end;
        end;
      end;
    finally
      CloseKey;
      Free;
    end;
  end;
  //sortieren
  Res.Sorted := False;
  Res.Sort;
  Result := Res;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TMainWindow.Log(Text: string; ShowAsMessageBox: boolean = False;
  AddToInfoMemo: boolean = False; AddToLogMemo: boolean = True);
begin
  if AddToLogMemo then
  begin
    LogMemo.Lines.Add(Text);
    LogMemo.Perform(EM_LineScroll, 0, LogMemo.Lines.Count - 1);
  end;
  if AddToInfoMemo = True then
  begin
    InfoMemo.Lines.Add(Text);
    InfoMemo.Perform(EM_LineScroll, 0, -InfoMemo.Lines.Count - 1);
  end;
  if ShowAsMessageBox = True then
    Application.MessageBox(PChar(Text), 'UpdateLoader - Hinweis',
      MB_OK + MB_IconExclamation);
end;

procedure TMainWindow.FlashHint(Show: boolean; Text: string = ''; Speed: integer = 500);
begin
  HintLabel.Caption := Text;
  HintLabel.Left    := round((InfoGroupBox.Width - HintLabel.Width) / 2);
  HintLabel.Visible := Show;
  HintFlashTimer.Interval := Speed;
  HintFlashTimer.Enabled := Show;
end;

procedure TMainWindow.ShowHexBuffer(var StringGrid: TStringGrid;
  var Buffer: TBuffer; var BufferLastAddress: longint;
  ShowCompleteBuffer: boolean = False);
//Hex-Buffer ausgeben
var
  I, BufferSize: longint;
  ByteCount:     integer;
  AddressText, LineContentHex, LineContentChr: string;
begin

  ByteCount := 1;

  if ShowCompleteBuffer = True then
    BufferSize := high(Buffer)
  else
    BufferSize := BufferLastAddress;

  if BufferSize > high(Buffer) then
    BufferSize := high(Buffer);

  //Ausgabe vorbereiten
  StringGrid.RowCount     := 1;
  StringGrid.ColWidths[0] := 80;
  StringGrid.ColWidths[1] := 195;
  StringGrid.ColWidths[2] := 20;
  StringGrid.ColWidths[3] := 210;
  StringGrid.ColWidths[4] := 150;
  StringGrid.Hide;         //Grid ausblenden, da sonst die Ausgabe verzgert wird

  //Statusanzeige
  ShowProgramProgressBar.Position := 0;
  ShowProgramProgressBar.Max      := BufferSize;

  for I := low(Buffer) to BufferSize do
  begin

    //Buffer-Inhalt in Zeichen umwandeln
    LineContentHex := LineContentHex + copy(BufferToHexStr(Buffer[I]), 2, 2) + ' ';
    LineContentChr := LineContentChr + chr(Buffer[I]);

    //Nach 16 Zeichen ausgeben in Grid, oder ausgeben wenn letztes Zeichen erreicht
    if (ByteCount = 16) or (I = BufferSize) then
    begin
      //Aktuelle Zeilenadresse
      AddressText := IntToHex(I - 15, 6) + ')';

      StringGrid.cells[0, StringGrid.RowCount - 1] := AddressText;
      StringGrid.cells[2, StringGrid.RowCount - 1] := '-';
      StringGrid.cells[4, StringGrid.RowCount - 1] := LineContentChr;

      //Abgekrzte Zeilen verarbeiten
      if length(LineContentHex) >= 24 then
      begin
        StringGrid.cells[1, StringGrid.RowCount - 1] :=
          copy(LineContentHex, 1, 24);
        StringGrid.cells[3, StringGrid.RowCount - 1] :=
          copy(LineContentHex, 25, length(LineContentHex));
      end
      else
      begin
        StringGrid.cells[3, StringGrid.RowCount - 1] :=
          copy(LineContentHex, 1, length(LineContentHex));
      end;

      StringGrid.RowHeights[StringGrid.RowCount - 1] := 18;
      StringGrid.RowCount := StringGrid.RowCount + 1;

      LineContentHex := '';
      LineContentChr := '';
      ByteCount      := 0;
    end;

    ByteCount := ByteCount + 1;

    ShowProgramProgressBar.Position := I;

    Application.ProcessMessages;

  end;

  ProgramGrid.RowCount := ProgramGrid.RowCount - 1;
  StringGrid.Show;

  ShowProgramProgressBar.Position := 0;

end;

{$ENDREGION}
////////////////////////////////////////////////////////////////////////////////
{$REGION 'Form-Aktionen und Checkbox-Clicks, Datei-Auswahl'}
procedure TMainWindow.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(FIni) then
  begin
    FIni.WriteString('Updater', 'BaudComboBox', BaudComboBox.Text);
    FIni.WriteString('Updater', 'ComComboBox', copy(ComComboBox.Text, 1, 4));

    FIni.WriteString('Updater', 'PasswordEdit', PasswordEdit.Text);
    FIni.WriteString('Updater', 'FilenameInEdit', FilenameInEdit.Text);

    FIni.WriteBool('Updater', 'ProgramChkBox', ProgramChkBox.Checked);
    FIni.WriteBool('Updater', 'VerifyChkBox', VerifyChkBox.Checked);
    FIni.WriteBool('Updater', 'ShowProgramChkBox', ShowProgramChkBox.Checked);
    FIni.WriteBool('Updater', 'ShowCompleteMemoryChkBox',
      ShowCompleteMemoryChkBox.Checked);

    FIni.UpdateFile;
    FIni.Free;
  end;
end;

procedure TMainWindow.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose := not InProgrammingMode;
end;

procedure TMainWindow.FormCreate(Sender: TObject);
begin
  MainWindow.Left := round((Screen.Width - MainWindow.Width) / 2);
  MainWindow.Top  := round((Screen.Height - MainWindow.Height) / 2);

  ReceiveMode   := 0;
  TxBufferEmpty := False;
  ConnectionCRC := 0;
end;

procedure TMainWindow.FormShow(Sender: TObject);
var
  I: integer;
  ComPorts: TStrings;
  SelectedPort: string;
begin
  ComPorts := GetComNames;

  FIni := TMemIniFile.Create(ExtractFilePath(Application.ExeName) + 'Settings.ini');

  PasswordEdit.Text := FIni.ReadString('Updater', 'PasswordEdit', 'Peda');

  ProgramChkBox.Checked := FIni.ReadBool('Updater', 'ProgramChkBox', True);
  VerifyChkBox.Checked  := FIni.ReadBool('Updater', 'VerifyChkBox', True);

  BaudComboBox.ItemIndex := BaudComboBox.Items.IndexOf(
    FIni.ReadString('Updater', 'BaudComboBox', '115200'));
  SelectedPort := FIni.ReadString('Updater', 'ComComboBox', 'COM1');

  ComComboBox.Text := ComPorts[0];
  for I := 0 to ComPorts.Count - 1 do
  begin
    ComComboBox.Items.Add(ComPorts[I]);
    if SelectedPort = copy(ComPorts[I], 1, 4) then
      ComComboBox.ItemIndex := ComComboBox.Items.IndexOf(ComPorts[I]);
  end;

  //Nach Ablauf des Timers (10ms) Dateiname setzen, damit die GUI zur Statusanzeige verfgbar ist
  StartupTimer.Enabled := True;
end;

procedure TMainWindow.StartupTimerTimer(Sender: TObject);
begin
  StartupTimer.Enabled := False;

  ShowProgramChkBox.Checked := FIni.ReadBool('Updater', 'ShowProgramChkBox', True);
  ShowCompleteMemoryChkBox.Checked :=
    FIni.ReadBool('Updater', 'ShowCompleteMemoryChkBox', False);

  FilenameInEdit.Text := FIni.ReadString('Updater', 'FilenameInEdit', '');
end;

////////////////////////////////////////////////////////////////////////////////

procedure TMainWindow.HintFlashTimerTimer(Sender: TObject);
begin
  HintLabel.Visible := not HintLabel.Visible;
end;

procedure TMainWindow.ProgramChkBoxClick(Sender: TObject);
begin
  ProgramProgressBar.Enabled := ProgramChkBox.Checked;
  BufferProgressBar.Enabled  := ProgramChkBox.Checked;
end;

procedure TMainWindow.ProgramGridDrawCell(Sender: TObject; ACol, ARow: integer;
  Rect: TRect; State: TGridDrawState);
begin
  case ACol of
    0:
    begin
      ProgramGrid.Canvas.Font.Color := RGB(128, 0, 0);
    end;
    4:
    begin
      ProgramGrid.Canvas.Font.Color := RGB(128, 0, 128);
    end;
    else
    begin
      ProgramGrid.Canvas.Font.Color := RGB(0, 0, 128);
    end;
  end;
  ProgramGrid.Canvas.TextOut(Rect.Left + 2, Rect.Top + 2, ProgramGrid.Cells[ACol, ARow]);
  ProgramGrid.Canvas.TextRect(Rect, Rect.Left + 2, Rect.Top + 2,
    ProgramGrid.Cells[ACol, ARow]);
end;

procedure TMainWindow.VerifyChkBoxClick(Sender: TObject);
begin
  VerifyProgressBar.Enabled := VerifyChkBox.Checked;
end;

procedure TMainWindow.ShowCompleteMemoryChkBoxClick(Sender: TObject);
begin
  //Programm anzeigen
  if (ShowProgramChkBox.Checked = True) and (StartBtn.Enabled = True) then
    ShowHexBuffer(ProgramGrid, ProgramBuffer, BufferLastAddress,
      ShowCompleteMemoryChkBox.Checked);
end;

procedure TMainWindow.FilenameInEditChange(Sender: TObject);
var
  ReadResult: integer;
begin
  if FileExists(FilenameInEdit.Text) then
  begin

    //Hex-Datei einlesen
    ReadResult := LoadHexFile(FilenameInEdit.Text, ProgramBuffer,
      MaxFlashSize, BufferLastAddress);
    if ReadResult = 1 then
    begin
      //Programm anzeigen
      if ShowProgramChkBox.Checked = True then
        ShowHexBuffer(ProgramGrid, ProgramBuffer, BufferLastAddress,
          ShowCompleteMemoryChkBox.Checked);

      StartBtn.Enabled := True;

      Log('Programmdatei "' + FilenameInEdit.Text + '" geladen');
    end
    else
    begin
      StartBtn.Enabled := False;
      case ReadResult of
        -5:
          Log('Eintrag in Datei doppelt vorhanden', True);
        -4:
          Log('Hex-Datei grer als Flash-Speicher (' + IntToStr(
            MaxFlashSize) + ' Byte)', True);
        -3:
          Log('Prfsumme der Hex-Datei ungltig', True);
        -2:
          Log('Ungltiger Record in Hex-Datei (unbekannte Funktion)', True);
        -1:
          Log('Ungltiges Dateiformat (kein Intel-Hex)', True);
      end;
    end;
  end
  else
  begin
    Log('Datei "' + FilenameInEdit.Text + '" nicht gefunden!', True, False);
    StartBtn.Enabled := False;
  end;
end;

procedure TMainWindow.ShowProgramChkBoxClick(Sender: TObject);
begin
  ProgramGroupBox.Visible := ShowProgramChkBox.Checked;
  if ShowProgramChkBox.Checked = True then
  begin
    MainWindow.Height := MainWindow.Height + ProgramGroupBox.Height;
  end
  else
  begin
    MainWindow.Height := MainWindow.Height - ProgramGroupBox.Height;
  end;
  MainWindow.Left := round((Screen.Width - MainWindow.Width) / 2);
  MainWindow.Top  := round((Screen.Height - MainWindow.Height) / 2);

  ShowCompleteMemoryChkBox.Enabled := ShowProgramChkBox.Checked;

  //Programm anzeigen
  if (StartBtn.Enabled = True) and (ShowProgramChkBox.Checked = True) then
    ShowHexBuffer(ProgramGrid, ProgramBuffer, BufferLastAddress,
      ShowCompleteMemoryChkBox.Checked);
end;

////////////////////////////////////////////////////////////////////////////////

procedure TMainWindow.SelectInBtnClick(Sender: TObject);
begin
  OpenDialog.FileName := FilenameInEdit.Text;
  if OpenDialog.Execute then
  begin
    FilenameInEdit.Text := OpenDialog.FileName;
  end;
end;

{$ENDREGION}
////////////////////////////////////////////////////////////////////////////////
{$REGION 'Comport-Events'}
procedure TMainWindow.ComPortTxEmpty(Sender: TObject);
begin
  TxBufferEmpty := True;
end;

procedure TMainWindow.ComPortRxBuf(Sender: TObject; const Buffer: PAnsiChar;
  Count: integer);
var
  I: integer;
  Command: char;
  SkipChar: boolean;
begin
  for I := 0 to Count - 1 do
  begin
    Command := Buffer[I];

    CommandReceived := True;

    if Command = HexToStr(Bootloader_FAIL) then
      Log('Kommando "' + LastCommand + '" fehlgeschlagen');
    if Command = HexToStr(Bootloader_BADCOMMAND) then
      Log('Kommando "' + LastCommand + '" nicht untersttzt');

    case ReceiveMode of
      1:  //Warten auf Connect/OneWire
      begin
        if Command = HexToStr(Bootloader_CONNECT) then
          Connected := True;
        if Command = copy(PasswordEdit.Text, 1, 1) then
          OneWire := True;
      end;
      2:  //Success abfragen
      begin
        if Command = HexToStr(Bootloader_SUCCESS) then
          ConnectionSuccess := True;
      end;
      3://CRC-Status
      begin
        if Command = HexToStr(Bootloader_BADCOMMAND) then
          CRCStatus := -2;
        if Command = HexToStr(Bootloader_FAIL) then
          CRCStatus := -1;
        if Command = HexToStr(Bootloader_SUCCESS) then
          CRCStatus := 1;
      end;
      4: //Zeichenkette empfangen
      begin
        if Command = HexToStr(Bootloader_ANSWER) then
          ReceiveAnswer := True;

        if ReceiveAnswer = True then
        begin
          SkipChar := False;
          if (Command = HexToStr('$03')) or (Command = HexToStr('$04')) or
            (Command = HexToStr(Bootloader_FAIL)) or
            (Command = HexToStr(Bootloader_SUCCESS)) or
            (Command = HexToStr(Bootloader_ANSWER)) then
            SkipChar := True;

          if SkipChar = False then
          begin
            if length(inttohex(Ord(Command), 0)) = 1 then
              Answer := Answer + '0';
            Answer := Answer + inttohex(Ord(Command), 0);
          end;

          if Command = HexToStr(Bootloader_FAIL) then
          begin
            AnswerResult   := -2;
            ReceiveAnswer  := False;
            AnswerReceived := True;
          end;
          if Command = HexToStr(Bootloader_SUCCESS) then
          begin
            AnswerResult   := 1;
            ReceiveAnswer  := False;
            AnswerReceived := True;
          end;

        end;
      end;
      5: //Continue
      begin
        if Command = HexToStr(Bootloader_CONTINUE) then
          ContinueProgram := True;
      end;
      6: //Programmieren - Success
      begin
        if Command = HexToStr(Bootloader_SUCCESS) then
          ProgSuccess := True;
      end;
      7: //Verify unterstzt
      begin
        if Command = HexToStr(Bootloader_BADCOMMAND) then
          SupportsVerify := False;
      end;
      8: //Ergebnis von Verify
      begin
        if Command = HexToStr(Bootloader_SUCCESS) then
          VerifySuccess := 1;
        if Command = HexToStr(Bootloader_FAIL) then
          VerifySuccess := -1;
      end;
    end;
  end;
end;

{$ENDREGION}
////////////////////////////////////////////////////////////////////////////////
{$REGION 'Byte-bertragungsfunktionen, CRC-Berechnung'}

procedure TMainWindow.CalculateCRC(Data: byte; var CRC: word);
var
  I: integer;
begin
  for I := 0 to 7 do
  begin
    if ((Data and $01) xor (CRC and $0001) <> 0) then
    begin
      CRC := CRC shr 1;
      CRC := CRC xor $A001;
    end
    else
      CRC := CRC shr 1;
    Data := Data shr 1;
  end;
end;

function TMainWindow.SendChar(Transmit: char; TxWaitTime: Dword;
  WaitUntilTx: boolean = True): boolean;
  //Sendet ein Zeichen
  //Rckgabe:
  //false=Timeout
  //true=OK
var
  Timeout: Dword;
begin
  Result := False;
  TxBufferEmpty := False;

  //Zeichen senden
  ComPort.WriteStr(Transmit);

  if WaitUntilTx = True then
  begin
    Timeout := GetTickCount;
    repeat
      if TxBufferEmpty = True then
      begin
        Timeout := 0;
        Result  := True;
      end;
      Application.ProcessMessages;
    until (GetTickCount - Timeout) > TxWaitTime;
  end
  else
    Result := True;

  //CRC berechnen
  CalculateCRC(Ord(Transmit), ConnectionCRC);
end;

function TMainWindow.SendCommand(HexCommand: string; TxWaitTime: Dword;
  WaitUntilTx: boolean = True): boolean;
begin
  Result      := False;
  LastCommand := HexCommand;
  if SendChar(HexToChar(Bootloader_COMMAND), TxWaitTime, WaitUntilTx) = False then
    exit;
  Result := SendChar(HexToChar(HexCommand), TxWaitTime, WaitUntilTx);
end;

function TMainWindow.SendStr(Text: string; TxWaitTime: Dword;
  WaitUntilTx: boolean = True): boolean;
var
  I: integer;
begin
  Result := False;
  for I := 1 to length(Text) do
  begin
    Result := SendChar(Text[I], TxWaitTime, WaitUntilTx);
    if Result = False then
      exit;
  end;
end;

{$ENDREGION}
////////////////////////////////////////////////////////////////////////////////
{$REGION 'Hilfsfunktionen zum Updaten (Datei lesen, Verbinden etc.)'}

function TMainWindow.LoadHexFile(Filename: string; var Buffer: TBuffer;
  BufferSize: longint; var BufferLastAddress: longint): integer;
  //Hex-Datei (Intel Format) in Puffer einlesen
  //Rckgabe:
  //-5=Zeile doppelt vorhanden
  //-4=Datei grer als Buffer
  //-3=Prfsumme ungltig
  //-2=Ungltiger Record
  //-1=Ungltige Datei
  //1=OK u. Dateiende
var
  HexFile: textfile;
  ReadLine, Temp: string;
  Checksum, RecordType, Data, LineChecksum: byte;
  ByteCount, DataLength: integer;
  Address, SegmentAdress, I: longint;
  SegmentedAdress: boolean;
begin

  Result := 0;
  LineChecksum := 0;
  SegmentedAdress := False;
  SegmentAdress := 0;
  BufferLastAddress := 0;

  //Buffer vorbereiten & leeren
  SetLength(Buffer, BufferSize);
  for I := low(Buffer) to high(Buffer) do
    Buffer[I] := StrToInt('$FF');

  try
    AssignFile(Hexfile, Filename);
    Reset(Hexfile);

    while not EOF(Hexfile) do
    begin
      ReadLn(Hexfile, ReadLine);

      if ReadLine[1] = ':' then //gltige Zeile?
      begin
        Checksum := 0;

        //Datensatz-Lnge bestimmen
        Temp     := '$' + copy(ReadLine, 2, 2);
        DataLength := StrToInt(Temp);
        Checksum := Checksum + DataLength;

        //Addresse bestimmen
        Temp     := '$' + copy(ReadLine, 4, 4);
        Address  := StrToInt(Temp);
        Checksum := Checksum + lo(Address) + hi(Address);

        if SegmentedAdress = True then
          //bei segmentierter Adresse: Adresse auf Segment-Adresse addieren
          Address := Address + SegmentAdress;

        //Record-Typ bestimmen
        Temp     := '$' + copy(ReadLine, 8, 2);
        RecordType := StrToInt(Temp);
        Checksum := Checksum + RecordType;

        case RecordType of
          0:  //Datenblock
          begin
            if (Address + DataLength - 1) <= BufferSize then
              //Passt aktueller Datenblock noch in den Speicher?
            begin
              ByteCount := 0;
              while ByteCount < DataLength do
              begin
                //Datenbyte auslesen
                Temp     := '$' + copy(ReadLine, 10 + 2 * ByteCount, 2);
                Data     := StrToInt(Temp);
                Checksum := Checksum + Data;
                //Prfen, ob Buffer bereits gefllt (Zeile kommt in Datei doppelt vor)
                if Buffer[Address + ByteCount] <> StrToInt('$FF') then
                  if Buffer[Address + ByteCount] = Data then
                  begin
                    Result := -5;
                    break;
                  end;
                //Byte in Buffer kopieren
                Buffer[Address + ByteCount] := Data;
                //nchstes Byte
                ByteCount := ByteCount + 1;
              end;
              if Result <> 0 then
                break;

              Temp := '$' + copy(ReadLine, 10 + 2 * ByteCount, 2);
              LineChecksum := StrToInt(Temp);

              //Letzte benutze Adresse im Buffer mitzhlen
              if BufferLastAddress <= Address + ByteCount - 1 then
                BufferLastAddress := Address + ByteCount - 1;

            end
            else
            begin
              Result := -4;
              break;
            end;
          end;
          1:  //Dateiende
          begin
            Temp   := '$' + copy(ReadLine, 10, 2);
            LineChecksum := StrToInt(Temp);
            Result := 1;
          end;
          2:  //Segment-Adresse
          begin
            SegmentedAdress := True;

            //Adresse auslesen
            Temp := '$' + copy(ReadLine, 10, 4);
            SegmentAdress := StrToInt(Temp) * 16;

            //Zeilen-Prfsumme
            Temp := '$' + copy(ReadLine, 4, 2);
            LineChecksum := StrToInt(Temp);
          end;
          else //Andere Angabe -> ungltig
          begin
            Result := -2;
            break;
          end;
        end;

        //Prfsumme
        Data := LineChecksum + Checksum;
        if (Data <> 0) then
        begin
          if Result >= 0 then
            Result := -3;
          break;
        end;

      end
      else
      begin
        Result := -1;
        break;
      end;

      Application.ProcessMessages;

    end;

    CloseFile(HexFile);
  except

  end;
end;

function TMainWindow.Connect: integer;
  //Verbindung aufbauen
  //Rckgabe:
  //-2=Timeout
  //-1=Versuche berschritten
  //1=OK
var
  I, CharCount: integer;
  SendResult:   boolean;
  Timeout:      dword;
begin

  OneWire   := False;
  Connected := False;
  ConnectionSuccess := False;

  CharCount := 1;

  ConnectProgressBar.Max := round(ConnectTries * (length(PasswordEdit.Text) + 1));

  for I := 0 to round(ConnectTries * (length(PasswordEdit.Text) + 1)) do
  begin
    ReceiveMode := 1;

    //Passwort zeichenweise senden
    if CharCount < length(PasswordEdit.Text) + 1 then
      SendResult := SendChar(PasswordEdit.Text[CharCount], SendTimeout)
    else
    begin
      SendResult := SendChar(HexToChar(Bootloader_PASSWORDAPPEND), SendTimeout);
      CharCount  := 0;
    end;
    CharCount := CharCount + 1;

    if SendResult = False then
    begin
      Result := -2;
      exit;
    end;

    Timeout := GetTickCount;
    repeat
      if Connected = True then
        Timeout := 0;
      Application.ProcessMessages;
    until (GetTickCount - Timeout) > ConnectTimeout;

    if Connected = True then
    begin
      ReceiveMode := 2;

      if SendChar(HexToChar(Bootloader_COMMAND), SendTimeout) = False then
      begin
        Result := -2;
        exit;
      end;

      Timeout := GetTickCount;
      repeat
        if ConnectionSuccess = True then
          Timeout := 0;
        Application.ProcessMessages;
      until (GetTickCount - Timeout) > GeneralTimeout;

      break;
    end;

    if (ConnectionSuccess = True) and (Connected = True) then
      break;

    ConnectProgressBar.Position := I + 1;

    Application.ProcessMessages;

  end;

  if (ConnectionSuccess = True) and (Connected = True) then
    Result := 1
  else
  if I = round(ConnectTries * (length(PasswordEdit.Text) + 1)) + 1 then
    Result := -1
  else
    Result := -2;

end;

function TMainWindow.CheckCRC: integer;
  //CRC-Prfsumme an Baustein senden und Ergebnis prfen
  //Rckgabe:
  //-2=CRC nicht untersttzt
  //-1=CRC-Fehler
  //1=OK
var
  Timeout: dword;
begin

  ReceiveMode := 3;
  CRCStatus   := 0;

  //Kommando senden
  SendCommand(Bootloader_CHECK_CRC, SendTimeout);

  Timeout := GetTickCount;
  repeat
    if CRCStatus <> 0 then
      Timeout := 0;
    Application.ProcessMessages;
  until (GetTickCount - Timeout) > GeneralTimeout;

  if CRCStatus = -2 then
  begin
    Result := -2;
    exit;
  end
  else
    CRCSupported := True;

  CRCStatus := 0;

  //Prfsumme senden
  SendStr(chr(lo(ConnectionCRC)) + chr(hi(ConnectionCRC)), SendTimeout);

  Timeout := GetTickCount;
  repeat
    if CRCStatus <> 0 then
      Timeout := 0;
    Application.ProcessMessages;
  until (GetTickCount - Timeout) > GeneralTimeout;

  Result := CRCStatus;

end;

function TMainWindow.ReadVal(Command: string; Timeout: dword): integer;
  //Zeichenkette zwischen Answer und Success empfangen
  //Rckgabe:
  //-2=Command Fail
  //-1=Timeout
  //1=OK
var
  StartTime: dword;
begin

  ReceiveMode := 4;
  ReceiveAnswer := False;
  AnswerReceived := False;
  AnswerResult := 0;
  Answer := '';

  SendCommand(Command, SendTimeout);

  StartTime := GetTickCount;
  repeat
    if AnswerReceived = True then
      Timeout := 0;
    Application.ProcessMessages;
  until (GetTickCount - StartTime) > Timeout;

  if AnswerReceived = False then
    Result := -1  //-1=Timeout
  else
    Result := AnswerResult; //Resultat vom Empfangen (1=OK, -2 =Fail)
end;

function TMainWindow.ReadInfos: boolean;
var
  ReadResult: integer;
begin
  Result := True;

  //Revision
  ReadResult := ReadVal(Bootloader_REVISION, ReadValTimeout);

  Log('Bootloader-Version: ' + HexToVer(Answer), False, True, False);
  ConnectProgressBar.Position := ConnectProgressBar.Position + 1;
  if (ReadResult <> 1) or (HexToVer(Answer) <> ValidBootloaderVersion) then
  begin
    Result := False;
    if HexToVer(Answer) <> ValidBootloaderVersion then
    begin
      Log('Bootloder-Version "' + HexToVer(Answer) +
        '" wird nicht untersttzt (unterstzte Version: "' +
        ValidBootloaderVersion + '")', True);
    end;
  end;

  //Signatur
  ReadResult := ReadVal(Bootloader_SIGNATURE, ReadValTimeout);

  Log('Baustein: ' + HexToDevice(Answer), False, True, False);
  ConnectProgressBar.Position := ConnectProgressBar.Position + 1;
  if (ReadResult <> 1) or (HexToDevice(Answer) = DefaultSignature) then
  begin
    Result := False;
    if HexToDevice(Answer) = DefaultSignature then
    begin
      Log('Baustein "' + DefaultSignature + '" nicht beschreibbar', True);
    end;
  end;

  //Buffergre
  ReadResult := ReadVal(Bootloader_BUFFSIZE, ReadValTimeout);

  Buffersize := HexToInt(Answer);

  Log('Buffergre: ' + IntToStr(HexToInt(Answer)) + ' Byte', False, True, False);
  ConnectProgressBar.Position := ConnectProgressBar.Position + 1;
  if (ReadResult <> 1) then
  begin
    Result := False;
  end;

  //Flashgre
  ReadResult := ReadVal(Bootloader_USERFLASH, ReadValTimeout);

  Flashsize := HexToInt(Answer);

  Log('Flash: ' + IntToStr(HexToInt(Answer)) + ' Byte', False, True, False);
  ConnectProgressBar.Position := ConnectProgressBar.Position + 1;
  if (ReadResult <> 1) or (HexToInt(Answer) > MaxFlashSize) then
  begin
    Result := False;
    if (HexToInt(Answer) > MaxFlashSize) then
    begin
      Log('Flash-Gre vom Baustein (' + IntToStr(HexToInt(Answer)) +
        ' Byte) berschreitet Obergrenze von diesem Tool (' +
        IntToStr(MaxFlashSize) + ' Byte)', True);
    end;
  end;
end;

function TMainWindow.VerifyDevice: integer;
  //Daten berprfen
  //Rckgabe:
  //-3=Reagiert nicht
  //-2=Fehler
  //-1=Verify nicht implementiert
  //1=OK
var
  I: integer;
  Timeout: dword;
begin

  ReceiveMode     := 7;
  SupportsVerify  := True;
  CommandReceived := False;

  Log('berprfe ' + IntToStr(BufferLastAddress) + ' von ' +
    IntToStr(FlashSize) + ' Bytes Flash', False, True);

  VerifyProgressBar.Max      := BufferLastAddress;
  VerifyProgressBar.Position := 0;

  //Verify starten
  SendCommand(Bootloader_VERIFY, SendTimeout);

  Timeout := GetTickCount;
  repeat
    if CommandReceived = True then
      Timeout := 0;
    Application.ProcessMessages;
  until (GetTickCount - Timeout) > GeneralTimeout;

  if SupportsVerify = True then
  begin
    for I := low(ProgramBuffer) to BufferLastAddress do
      //Anfang Buffer bis letzte benutzte Adresse
    begin
      //Escape-Shift
      if (BufferToHexStr(ProgramBuffer[i]) = '$13') or
        (BufferToHexStr(ProgramBuffer[i]) = Bootloader_ESCAPE) then
      begin
        SendChar(HexToChar(Bootloader_ESCAPE), SendTimeout);
        SendChar(ShiftBufferEscape(ProgramBuffer[i]), SendTimeout);
      end
      else
        SendChar(BufferToChar(ProgramBuffer[i]), SendTimeout);

      Application.ProcessMessages;
      VerifyProgressBar.Position := i;
    end;

    //Verify beenden
    ReceiveMode   := 8;
    VerifySuccess := 0;

    SendStr(HexToStr(Bootloader_ESCAPE) + HexToStr(Bootloader_PROGEND), SendTimeout);

    Timeout := GetTickCount;
    repeat
      if VerifySuccess <> 0 then
        Timeout := 0;
      Application.ProcessMessages;
    until (GetTickCount - Timeout) > VerifyTimeout;

    if VerifySuccess = 1 then
      Result := 1
    else
    if VerifySuccess = -1 then
      Result := -2
    else
      Result := -3;
  end
  else
    Result := -1;
end;

function TMainWindow.ProgramDevice: integer;
  //Programm an Device senden
  //Rckgabe:
  //-2=keine Success-Meldung
  //-1=Reagiert nicht
  //1=OK
var
  I, WrittenBuffer: integer;
  Timeout: dword;
begin

  ReceiveMode := 5;

  Log('Programmiere ' + IntToStr(BufferLastAddress) + ' von ' +
    IntToStr(FlashSize) + ' Bytes Flash', False, True);

  ProgramProgressBar.Max     := BufferLastAddress;
  ProgramProgressBar.Position := 0;
  BufferProgressBar.Max      := BufferSize;
  BufferProgressBar.Position := 0;

  //Programmieren starten
  SendCommand(Bootloader_PROGRAM, SendTimeout);

  WrittenBuffer := BufferSize;

  for I := low(ProgramBuffer) to BufferLastAddress do
    //Anfang Buffer bis letzte benutzte Adresse
  begin
    //Escape-Shift
    if (BufferToHexStr(ProgramBuffer[i]) = '$13') or
      (BufferToHexStr(ProgramBuffer[i]) = Bootloader_ESCAPE) then
    begin
      SendChar(HexToChar(Bootloader_ESCAPE), SendTimeout);
      SendChar(ShiftBufferEscape(ProgramBuffer[i]), SendTimeout);
    end
    else
      SendChar(BufferToChar(ProgramBuffer[i]), SendTimeout);


    //Puffer voll -> warten auf "Continue"
    WrittenBuffer := WrittenBuffer - 1;
    if WrittenBuffer = 0 then
    begin
      WrittenBuffer := BufferSize;
      ContinueProgram := False;
      Timeout := GetTickCount;
      repeat
        if ContinueProgram = True then
          Timeout := 0;
        Application.ProcessMessages;
      until (GetTickCount - Timeout) > ProgramTimeout;

      if ContinueProgram = False then
      begin
        Result := -1;
        exit;
      end;
    end;

    Application.ProcessMessages;
    ProgramProgressBar.Position := i;
    BufferProgressBar.Position  := WrittenBuffer;
  end;

  //Programmieren beenden
  ReceiveMode := 6;
  ProgSuccess := False;

  SendStr(HexToStr(Bootloader_ESCAPE) + HexToStr(Bootloader_PROGEND), SendTimeout);

  Timeout := GetTickCount;
  repeat
    if ProgSuccess = True then
      Timeout := 0;
    Application.ProcessMessages;
  until (GetTickCount - Timeout) > ProgramTimeout;

  if ProgSuccess = True then
    Result := 1
  else
    Result := -2;

end;

{$ENDREGION}
////////////////////////////////////////////////////////////////////////////////

procedure TMainWindow.StartBtnClick(Sender: TObject);
var
  StartUpdate, ConnectResult, CRCResult, ProgResult, VerifyResult, StartVerify: integer;
  StartTime: TDateTime;
  Temp:      string;
  WriteProgram, VerifyProgram, StartProgram: boolean;
begin

  WriteProgram  := ProgramChkBox.Checked;
  VerifyProgram := VerifyChkBox.Checked;

  if (WriteProgram = True) or (VerifyProgram = True) then
    //Prfen, ob es berhaupt was zu tun gibt
  begin

    //GUI-Ausgaben vorbereiten
    StartTime := TDateTime(now);
    InProgrammingMode := True;
    MainWindow.FormStyle := fsStayOnTop;

    StartBtn.Enabled      := False;
    StartBtn.Caption      := 'Bitte warten!';
    SelectInBtn.Enabled   := False;
    FilenameInEdit.Enabled := False;
    PasswordEdit.Enabled  := False;
    BaudComboBox.Enabled  := False;
    ComComboBox.Enabled   := False;
    ProgramChkBox.Enabled := False;
    VerifyChkBox.Enabled  := False;
    ShowProgramChkBox.Enabled := False;
    ShowCompleteMemoryChkBox.Enabled := False;
    InfoMemo.Clear;
    ConnectProgressBar.Position := 0;
    ProgramProgressBar.Position := 0;
    VerifyProgressBar.Position  := 0;
    BufferProgressBar.Position  := 0;

    StartProgram := False;

    //Hinweis fr Nutzer ausgeben
    StartUpdate := Application.MessageBox(
      'Warnung: Die Aktualisierung der Firmware kann nicht mehr rckgngig gemacht werden. Im Fehlerfall kann das Gert beschdigt werden.' + #10#13#10#13 + 'Update wirklich durchfhren?', 'UpdateLoader', MB_YesNo + MB_IconQuestion + MB_DefButton1);
    if StartUpdate = idYes then
    begin
      ShowMessage(
        'Schalten Sie nun das Gert aus und warten Sie, bis die Software Sie zum Wiedereinschalten auffordert.');

      //Verbindung aufbauen
      ComPort.Port     := copy(ComComboBox.Text, 1, 4);
      ComPort.BaudRate := StrToBaudRate(BaudComboBox.Text);
      ComPort.Open;

      //3 Sekunden warten, Hinweis ausgeben und weitere 1 Sekunde warten
      Sleep(3000);
      HintLabel.Visible := True;
      HintLabel.Caption := 'Gert wieder einschalten!';
      HintLabel.Left    := round((InfoGroupBox.Width - HintLabel.Width) / 2);
      Sleep(1000);

      ConnectProgressBar.Position := 1;

      //Verbinden
      ConnectResult := Connect;

      ConnectProgressBar.Position :=
        round(ConnectTries * (length(PasswordEdit.Text) + 1)) + 1;

      if ConnectResult = 1 then //Erfolgreich verbunden
      begin
        FlashHint(True, 'Update luft');

        if OneWire then
        begin
          Log('Verbunden: 1-Draht-Modus', False, True, False);
          ShowMessage('Dieser Modus wurde noch nicht getestet!!!');
        end
        else
          Log('Verbunden: 2-Draht-Modus', False, True, False);

        CRCSupported := False;    //Prfen, ob CRC verfgbar
        CheckCRC;

        ConnectProgressBar.Position := ConnectProgressBar.Position + 1;

        if CRCSupported = True then
          Log('CRC-Check: aktiviert', False, True, False)
        else
          Log('CRC-Check: deaktiviert', False, True, False);

        if ReadInfos = True then //Infos (Flashgre, ID, etc) auslesen
        begin

          CRCResult := CheckCRC;
          //Nach dem Info-Auslesen 1. CRC prfen, wenn hier bereits ein Fehler auftritt, Programmieren abbrechen 
          if (CRCResult = 1) or (CRCResult = -2) then
          begin
            if CRCSupported = True then
              Log('CRC-Prfsumme OK', False, True, False);

            //Programmieren
            if WriteProgram = True then
            begin
              ProgResult := ProgramDevice;
              BufferProgressBar.Position := BufferSize;
              if ProgResult = 1 then
              begin
                Log('Programmieren erfolgreich beendet');

                //Verify vorschlagen
                if VerifyProgram = False then
                begin
                  StartVerify :=
                    Application.MessageBox(
                    'Hinweise: Um die Funktionssicherheit zu erhhen, sollte die geschrieben Software berprft werden.'
                    + #10#13#10#13 + 'Jetzt berprfen?',
                    'UpdateLoader', MB_YesNo + MB_IconQuestion + MB_DefButton1);
                  if StartVerify = idYes then
                    VerifyProgram := True;
                end;

              end
              else
              begin
                VerifyProgram := False;
                WriteProgram  := False;
                case ProgResult of
                  -2:
                    Log('Programmieren fehlgeschlagen', True);
                  -1:
                    Log('Baustein reagiert nicht', True);
                  else
                    Log('Programmieren fehlgeschlagen', True);
                end;
              end;
            end;

            //berprfen
            if VerifyProgram = True then
            begin
              VerifyResult := VerifyDevice;
              if VerifyResult >= -1 then
              begin

                if VerifyResult = -1 then
                  Log('berprfen nicht mglich (nicht implementiert)')
                else
                  Log('Gespeichertes Programm korrekt bertragen');

              end
              else
              begin
                VerifyProgram := False;
                WriteProgram  := False;
                case VerifyResult of
                  -3:
                    Log('Baustein reagiert nicht', True);
                  -2:
                    Log('Gespeichertes Programm stimmt nicht mit Datei berein', True);
                  else
                    Log('Fehler beim berprfen', True);
                end;
              end;

            end;

            //Nach dem Programmieren: CRC berprfen

            CRCResult := CheckCRC;
            if (CRCResult = 1) or (CRCResult = -2) then
            begin
              if CRCSupported = True then
                Log('CRC-Prfsumme OK', False, True, False);

              //Wenn CRC-Summe OK und programmieren/berprfen OK: Programm starten
              if (WriteProgram = True) or (VerifyProgram = True) then
                StartProgram := True;
            end
            else
            begin
              case CRCResult of
                -1:
                  Log('CRC-Prfsumme stimmt nicht berein', True);
                else
                  Log('Prfsummenfehler', True);
              end;
            end;

          end
          else
          begin
            case CRCResult of
              -1:
                Log('CRC-Prfsumme stimmt nicht berein', True);
              else
                Log('Prfsummenfehler', True);
            end;
          end;

        end
        else
        begin
          Log('Baustein-Informationen fehlerhaft/Baustein nicht untersttzt');
        end;

        //Programmieren beenden
        SendCommand(Bootloader_PROGEND, SendTimeout);
        SendCommand(Bootloader_COMMAND, SendTimeout);
        //Programm starten
        if StartProgram = True then
          SendCommand(Bootloader_START, SendTimeout);

      end
      else
      begin
        case ConnectResult of
          -2:
            Log('Zeitberschreitung beim Senden', True);
          -1:
            Log('Maximale Anzahl Verbindungsversuche (' + IntToStr(ConnectTries) +
              ') berschritten', True);
          else
            Log('Fehler beim Herstellen der Verbindung', True);
        end;
      end;

    end;

    //Port schlieen
    ComPort.Close;

    //GUI-Ausgabe
    StartBtn.Enabled      := True;
    StartBtn.Caption      := 'Update starten';
    SelectInBtn.Enabled   := True;
    FilenameInEdit.Enabled := True;
    PasswordEdit.Enabled  := True;
    BaudComboBox.Enabled  := True;
    ComComboBox.Enabled   := True;
    ProgramChkBox.Enabled := True;
    VerifyChkBox.Enabled  := True;
    ShowProgramChkBox.Enabled := True;
    ShowCompleteMemoryChkBox.Enabled := True;
    ConnectProgressBar.Position := 0;
    ProgramProgressBar.Position := 0;
    VerifyProgressBar.Position := 0;
    BufferProgressBar.Position := 0;
    InProgrammingMode     := False;
    MainWindow.FormStyle  := fsNormal;

    DateTimeToString(Temp, 'hh:mm:ss', TDateTime(now) - StartTime);
    Log('Gesamtdauer: ' + Temp, False, True, False);

    FlashHint(False);

  end
  else
    ShowMessage('Der Baustein musst mindestens programmiert oder berprft werden.');

end;

end.
